import socket
import json
import re
from flask import Flask, render_template, redirect, flash, request, url_for


class GSTDException(Exception):
    pass

class GSTDClient(object):
    def __init__(self, addr='127.0.0.1', port=5000):
        self.addr = addr
        self.port = port

    def connect(self):
        self.sock = socket.socket()
        self.sock.connect((self.addr, self.port))

    def request(self, cmd, args=None):
        if args:
            cmd += ' ' + args
        print('<', cmd)
        self.sock.send(cmd.encode())
        resp = self.recv_object()
        print('>', resp)
        if resp['code'] != 0:
            raise GSTDException(resp)
        return resp['response']

    def recv_object(self):
        buf = ''
        decoder = json.JSONDecoder()

        while True:
            buf += self.sock.recv(4096).decode().replace('],{', '],') # wat
            try:
                obj, end = decoder.raw_decode(buf, 0)
                return obj
            except ValueError as exc:
                print(exc)
                pass

    def pipelines_state(self):
        return {
            p['name']: self.request('read', '/pipelines/%s/state' % (p['name'],))['value']
            for p in self.request('list_pipelines')['nodes']
        }

    def pipeline_elements(self, name):
        elements = self.request('read', '/pipelines/%s/elements' % (name,))['nodes']

        return {
            e['name']: self.request('read', '/pipelines/%s/elements/%s' % (name, e['name']))['element_properties']
            for e in elements
        }


app = Flask(__name__)
app.config['TEMPLATES_AUTO_RELOAD'] = True
app.config['SECRET_KEY'] = 'changeme'

def parse_access(access):
    return int(re.findall('\d+', access)[0])


@app.template_filter()
def access_writable(access):
    return bool(parse_access(access) & 2)


def gstd():
    c = GSTDClient()
    c.connect()
    return c


@app.route('/')
def index():
    c = gstd()
    return render_template('index.html', pipelines=c.pipelines_state())


@app.route('/pipelines/<name>/edit')
def pipeline_details(name):
    c = gstd()
    return render_template('pipeline_details.html',
                           elements=c.pipeline_elements(name), pipeline=name)


@app.route('/pipelines/<pipeline>/edit/<element>/<prop>')
def element_edit(pipeline, element, prop):
    c = gstd()
    try:
        c.request('element_set', ' '.join([pipeline, element, prop, request.args.get('value', '')]))
    except GSTDException as exc:
        flash(exc.args[0]['description'], 'danger')
    return redirect(url_for('pipeline_details', name=pipeline))


@app.route('/pipelines/<name>/<action>')
def pipeline_action(name, action):
    c = gstd()
    try:
        c.request('pipeline_%s' % action, name)
    except GSTDException as exc:
        flash(exc.args[0]['description'], 'danger')
    return redirect('/')
