"""Small example OSC server

This program listens to several addresses, and prints some information about
received packets.
"""
import argparse
import math
import threading
import time

from pythonosc import dispatcher
from pythonosc import osc_server
from pythonosc import udp_client

from snowmix import SnowmixClient

def snowmix_wrap(func, **kwargs):
    def wrapped(addr, *args):
        print(func, addr, *args)
        return func(*args)
    return wrapped


class PollerThread(threading.Thread):
    daemon = True
    def run(self):
        while True:
            try:
                for scene_id, state in self.client.tcl('SceneSetState'):
                    print(scene_id, state)
                    self.osc.send_message('/scene/state', [scene_id, float(state)])

                for feed_id, status in self.client.audio_status().items():
                    #print(status)
                    self.osc.send_message('/feed/state', [feed_id, status['state']])
                    for i, v in enumerate(status['rms'].split(',')):
                        #msg = '/feed/%s/rms_%s' % (feed_id, ['l', 'r'][i])
                        #print(msg, float(v))
                        #self.osc.send_message(msg, float(v))
                        msg = '/feed/rms_%s' % (['l', 'r'][i],)
                        self.osc.send_message(msg, [feed_id, float(v)])

                #print(list(self.client.call('audio feed status', 'STAT:'))[1:])
                #for s in self.client.scene_list():
                #    print(self.client.scene_info(s))
            except Exception as exc:
                print(exc)
                pass

            time.sleep(0.1)

if __name__ == "__main__":
  parser = argparse.ArgumentParser()
  parser.add_argument("--ip",
      default="127.0.0.1", help="The ip to listen on")
  parser.add_argument("--port",
      type=int, default=5005, help="The port to listen on")
  parser.add_argument("--status-ip",
      default="255.255.255.255", help="IP to report status on")
  parser.add_argument("--status-port",
      type=int, default=8080, help="The port to report status on")
  parser.add_argument("--snowmix-ip",
      default="10.8.0.95", help="Snowmix server IP")
  parser.add_argument("--snowmix-port", type=int, default=9999, help="Snowmix port")
  args = parser.parse_args()

  s = SnowmixClient(args.snowmix_ip, args.snowmix_port)
  dispatcher = dispatcher.Dispatcher()

  #dispatcher.map("/filter", print)
  dispatcher.map("/feed/volume", snowmix_wrap(s.audio_volume))
  dispatcher.map("/feed/mute", snowmix_wrap(s.audio_mute))
  dispatcher.map("/scene/fade", snowmix_wrap(s.scene_fade))
  dispatcher.map("/scene/cut", snowmix_wrap(s.scene_cut))

  poller = PollerThread()
  poller.osc = udp_client.SimpleUDPClient(args.status_ip, args.status_port, True)
  poller.client = s
  poller.start()

  server = osc_server.ThreadingOSCUDPServer(
      (args.ip, args.port), dispatcher)
  print("Serving on {}".format(server.server_address))
  server.serve_forever()
